@@ -1,15 +1,15 @@
+# Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
+
 import pytest
 
 import astroid
 from astroid import bases, nodes
-from astroid.const import PY37_PLUS
+from astroid.const import PY310_PLUS
 from astroid.exceptions import InferenceError
 from astroid.util import Uninferable
 
-if not PY37_PLUS:
-    pytest.skip("Dataclasses were added in 3.7", allow_module_level=True)
-
-
 parametrize_module = pytest.mark.parametrize(
     ("module",), (["dataclasses"], ["pydantic.dataclasses"], ["marshmallow_dataclass"])
 )
@@ -184,6 +184,7 @@ def test_inference_no_annotation(module: str):
     inferred = next(class_def.infer())
     assert isinstance(inferred, nodes.ClassDef)
     assert inferred.instance_attrs == {}
+    assert inferred.is_dataclass
 
     # Both the class and instance can still access the attribute
     for node in (klass, instance):
@@ -216,6 +217,7 @@ def test_inference_class_var(module: str):
     inferred = next(class_def.infer())
     assert isinstance(inferred, nodes.ClassDef)
     assert inferred.instance_attrs == {}
+    assert inferred.is_dataclass
 
     # Both the class and instance can still access the attribute
     for node in (klass, instance):
@@ -248,6 +250,7 @@ def test_inference_init_var(module: str):
     inferred = next(class_def.infer())
     assert isinstance(inferred, nodes.ClassDef)
     assert inferred.instance_attrs == {}
+    assert inferred.is_dataclass
 
     # Both the class and instance can still access the attribute
     for node in (klass, instance):
@@ -622,12 +625,12 @@ def test_init_attributes_from_superclasses(module: str):
     """
     )
     init = next(node.infer())
-    assert [a.name for a in init.args.args] == ["self", "arg0", "arg2", "arg1"]
+    assert [a.name for a in init.args.args] == ["self", "arg0", "arg1", "arg2"]
     assert [a.as_string() if a else None for a in init.args.annotations] == [
         None,
         "float",
-        "list",  # not str
         "int",
+        "list",  # not str
     ]
 
 
@@ -666,6 +669,7 @@ def test_annotated_enclosed_field_call(module: str):
     inferred = node.inferred()
     assert len(inferred) == 1 and isinstance(inferred[0], nodes.ClassDef)
     assert "attribute" in inferred[0].instance_attrs
+    assert inferred[0].is_dataclass
 
 
 @parametrize_module
@@ -683,3 +687,360 @@ def test_invalid_field_call(module: str) -> None:
     inferred = code.inferred()
     assert len(inferred) == 1
     assert isinstance(inferred[0], nodes.ClassDef)
+    assert inferred[0].is_dataclass
+
+
+def test_non_dataclass_is_not_dataclass() -> None:
+    """Test that something that isn't a dataclass has the correct attribute."""
+    module = astroid.parse(
+        """
+    class A:
+        val: field()
+
+    def dataclass():
+        return
+
+    @dataclass
+    class B:
+        val: field()
+    """
+    )
+    class_a = module.body[0].inferred()
+    assert len(class_a) == 1
+    assert isinstance(class_a[0], nodes.ClassDef)
+    assert not class_a[0].is_dataclass
+
+    class_b = module.body[2].inferred()
+    assert len(class_b) == 1
+    assert isinstance(class_b[0], nodes.ClassDef)
+    assert not class_b[0].is_dataclass
+
+
+def test_kw_only_sentinel() -> None:
+    """Test that the KW_ONLY sentinel doesn't get added to the fields."""
+    node_one, node_two = astroid.extract_node(
+        """
+    from dataclasses import dataclass, KW_ONLY
+    from dataclasses import KW_ONLY as keyword_only
+
+    @dataclass
+    class A:
+        _: KW_ONLY
+        y: str
+
+    A.__init__  #@
+
+    @dataclass
+    class B:
+        _: keyword_only
+        y: str
+
+    B.__init__  #@
+    """
+    )
+    if PY310_PLUS:
+        expected = ["self", "y"]
+    else:
+        expected = ["self", "_", "y"]
+    init = next(node_one.infer())
+    assert [a.name for a in init.args.args] == expected
+
+    init = next(node_two.infer())
+    assert [a.name for a in init.args.args] == expected
+
+
+def test_kw_only_decorator() -> None:
+    """Test that we update the signature correctly based on the keyword.
+
+    kw_only was introduced in PY310.
+    """
+    foodef, bardef, cee, dee = astroid.extract_node(
+        """
+    from dataclasses import dataclass
+
+    @dataclass(kw_only=True)
+    class Foo:
+        a: int
+        e: str
+
+
+    @dataclass(kw_only=False)
+    class Bar(Foo):
+        c: int
+
+
+    @dataclass(kw_only=False)
+    class Cee(Bar):
+        d: int
+
+
+    @dataclass(kw_only=True)
+    class Dee(Cee):
+        ee: int
+
+
+    Foo.__init__  #@
+    Bar.__init__  #@
+    Cee.__init__  #@
+    Dee.__init__  #@
+    """
+    )
+
+    foo_init: bases.UnboundMethod = next(foodef.infer())
+    if PY310_PLUS:
+        assert [a.name for a in foo_init.args.args] == ["self"]
+        assert [a.name for a in foo_init.args.kwonlyargs] == ["a", "e"]
+    else:
+        assert [a.name for a in foo_init.args.args] == ["self", "a", "e"]
+        assert [a.name for a in foo_init.args.kwonlyargs] == []
+
+    bar_init: bases.UnboundMethod = next(bardef.infer())
+    if PY310_PLUS:
+        assert [a.name for a in bar_init.args.args] == ["self", "c"]
+        assert [a.name for a in bar_init.args.kwonlyargs] == ["a", "e"]
+    else:
+        assert [a.name for a in bar_init.args.args] == ["self", "a", "e", "c"]
+        assert [a.name for a in bar_init.args.kwonlyargs] == []
+
+    cee_init: bases.UnboundMethod = next(cee.infer())
+    if PY310_PLUS:
+        assert [a.name for a in cee_init.args.args] == ["self", "c", "d"]
+        assert [a.name for a in cee_init.args.kwonlyargs] == ["a", "e"]
+    else:
+        assert [a.name for a in cee_init.args.args] == ["self", "a", "e", "c", "d"]
+        assert [a.name for a in cee_init.args.kwonlyargs] == []
+
+    dee_init: bases.UnboundMethod = next(dee.infer())
+    if PY310_PLUS:
+        assert [a.name for a in dee_init.args.args] == ["self", "c", "d"]
+        assert [a.name for a in dee_init.args.kwonlyargs] == ["a", "e", "ee"]
+    else:
+        assert [a.name for a in dee_init.args.args] == [
+            "self",
+            "a",
+            "e",
+            "c",
+            "d",
+            "ee",
+        ]
+        assert [a.name for a in dee_init.args.kwonlyargs] == []
+
+
+def test_dataclass_with_unknown_base() -> None:
+    """Regression test for dataclasses with unknown base classes.
+
+    Reported in https://github.com/PyCQA/pylint/issues/7418
+    """
+    node = astroid.extract_node(
+        """
+    import dataclasses
+
+    from unknown import Unknown
+
+
+    @dataclasses.dataclass
+    class MyDataclass(Unknown):
+        pass
+
+    MyDataclass()
+    """
+    )
+
+    assert next(node.infer())
+
+
+def test_dataclass_with_unknown_typing() -> None:
+    """Regression test for dataclasses with unknown base classes.
+
+    Reported in https://github.com/PyCQA/pylint/issues/7422
+    """
+    node = astroid.extract_node(
+        """
+    from dataclasses import dataclass, InitVar
+
+
+    @dataclass
+    class TestClass:
+        '''Test Class'''
+
+        config: InitVar = None
+
+    TestClass.__init__  #@
+    """
+    )
+
+    init_def: bases.UnboundMethod = next(node.infer())
+    assert [a.name for a in init_def.args.args] == ["self", "config"]
+
+
+def test_dataclass_with_default_factory() -> None:
+    """Regression test for dataclasses with default values.
+
+    Reported in https://github.com/PyCQA/pylint/issues/7425
+    """
+    bad_node, good_node = astroid.extract_node(
+        """
+    from dataclasses import dataclass
+    from typing import Union
+
+    @dataclass
+    class BadExampleParentClass:
+        xyz: Union[str, int]
+
+    @dataclass
+    class BadExampleClass(BadExampleParentClass):
+        xyz: str = ""
+
+    BadExampleClass.__init__  #@
+
+    @dataclass
+    class GoodExampleParentClass:
+        xyz: str
+
+    @dataclass
+    class GoodExampleClass(GoodExampleParentClass):
+        xyz: str = ""
+
+    GoodExampleClass.__init__  #@
+    """
+    )
+
+    bad_init: bases.UnboundMethod = next(bad_node.infer())
+    assert bad_init.args.defaults
+    assert [a.name for a in bad_init.args.args] == ["self", "xyz"]
+
+    good_init: bases.UnboundMethod = next(good_node.infer())
+    assert bad_init.args.defaults
+    assert [a.name for a in good_init.args.args] == ["self", "xyz"]
+
+
+def test_dataclass_with_multiple_inheritance() -> None:
+    """Regression test for dataclasses with multiple inheritance.
+
+    Reported in https://github.com/PyCQA/pylint/issues/7427
+    """
+    first, second, overwritten, overwriting, mixed = astroid.extract_node(
+        """
+    from dataclasses import dataclass
+
+    @dataclass
+    class BaseParent:
+        _abc: int = 1
+
+    @dataclass
+    class AnotherParent:
+        ef: int = 2
+
+    @dataclass
+    class FirstChild(BaseParent, AnotherParent):
+        ghi: int = 3
+
+    @dataclass
+    class ConvolutedParent(AnotherParent):
+        '''Convoluted Parent'''
+
+    @dataclass
+    class SecondChild(BaseParent, ConvolutedParent):
+        jkl: int = 4
+
+    @dataclass
+    class OverwritingParent:
+        ef: str = "2"
+
+    @dataclass
+    class OverwrittenChild(OverwritingParent, AnotherParent):
+        '''Overwritten Child'''
+
+    @dataclass
+    class OverwritingChild(BaseParent, AnotherParent):
+        _abc: float = 1.0
+        ef: float = 2.0
+
+    class NotADataclassParent:
+        ef: int = 2
+
+    @dataclass
+    class ChildWithMixedParents(BaseParent, NotADataclassParent):
+        ghi: int = 3
+
+    FirstChild.__init__  #@
+    SecondChild.__init__  #@
+    OverwrittenChild.__init__  #@
+    OverwritingChild.__init__  #@
+    ChildWithMixedParents.__init__  #@
+    """
+    )
+
+    first_init: bases.UnboundMethod = next(first.infer())
+    assert [a.name for a in first_init.args.args] == ["self", "ef", "_abc", "ghi"]
+    assert [a.value for a in first_init.args.defaults] == [2, 1, 3]
+
+    second_init: bases.UnboundMethod = next(second.infer())
+    assert [a.name for a in second_init.args.args] == ["self", "ef", "_abc", "jkl"]
+    assert [a.value for a in second_init.args.defaults] == [2, 1, 4]
+
+    overwritten_init: bases.UnboundMethod = next(overwritten.infer())
+    assert [a.name for a in overwritten_init.args.args] == ["self", "ef"]
+    assert [a.value for a in overwritten_init.args.defaults] == ["2"]
+
+    overwriting_init: bases.UnboundMethod = next(overwriting.infer())
+    assert [a.name for a in overwriting_init.args.args] == ["self", "_abc", "ef"]
+    assert [a.value for a in overwriting_init.args.defaults] == [1.0, 2.0]
+
+    mixed_init: bases.UnboundMethod = next(mixed.infer())
+    assert [a.name for a in mixed_init.args.args] == ["self", "_abc", "ghi"]
+    assert [a.value for a in mixed_init.args.defaults] == [1, 3]
+
+
+def test_dataclass_inits_of_non_dataclasses() -> None:
+    """Regression test for __init__ mangling for non dataclasses.
+
+    Regression test against changes tested in test_dataclass_with_multiple_inheritance
+    """
+    first, second, third = astroid.extract_node(
+        """
+    from dataclasses import dataclass
+
+    @dataclass
+    class DataclassParent:
+        _abc: int = 1
+
+
+    class NotADataclassParent:
+        ef: int = 2
+
+
+    class FirstChild(DataclassParent, NotADataclassParent):
+        ghi: int = 3
+
+
+    class SecondChild(DataclassParent, NotADataclassParent):
+        ghi: int = 3
+
+        def __init__(self, ef: int = 3):
+            self.ef = ef
+
+
+    class ThirdChild(NotADataclassParent, DataclassParent):
+        ghi: int = 3
+
+        def __init__(self, ef: int = 3):
+            self.ef = ef
+
+    FirstChild.__init__  #@
+    SecondChild.__init__  #@
+    ThirdChild.__init__  #@
+    """
+    )
+
+    first_init: bases.UnboundMethod = next(first.infer())
+    assert [a.name for a in first_init.args.args] == ["self", "_abc"]
+    assert [a.value for a in first_init.args.defaults] == [1]
+
+    second_init: bases.UnboundMethod = next(second.infer())
+    assert [a.name for a in second_init.args.args] == ["self", "ef"]
+    assert [a.value for a in second_init.args.defaults] == [3]
+
+    third_init: bases.UnboundMethod = next(third.infer())
+    assert [a.name for a in third_init.args.args] == ["self", "ef"]
+    assert [a.value for a in third_init.args.defaults] == [3]
