@@ -1,42 +1,18 @@
-# Copyright (c) 2006-2007, 2009-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
-# Copyright (c) 2013-2021 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Google, Inc.
-# Copyright (c) 2014 Eevee (Alex Munroe) <amunroe@yelp.com>
-# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
-# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
-# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
-# Copyright (c) 2017 rr- <rr-@sakuya.pl>
-# Copyright (c) 2017 Derek Gustafson <degustaf@gmail.com>
-# Copyright (c) 2018 Serhiy Storchaka <storchaka@gmail.com>
-# Copyright (c) 2018 brendanator <brendan.maginnis@gmail.com>
-# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2019-2021 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2019 Alex Hall <alex.mojaki@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2021 Tushar Sadhwani <86737547+tushar-deepsource@users.noreply.github.com>
-# Copyright (c) 2021 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 René Fritze <47802+renefritze@users.noreply.github.com>
-# Copyright (c) 2021 Federico Bond <federicobond@gmail.com>
-# Copyright (c) 2021 hippo91 <guillaume.peillex@gmail.com>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
 """tests for specific behaviour of astroid nodes
 """
+
+from __future__ import annotations
+
 import copy
 import os
-import platform
 import sys
 import textwrap
 import unittest
-from typing import Any, Optional
+from typing import Any
 
 import pytest
 
@@ -289,12 +265,6 @@ def func(param: Tuple):
         ast = abuilder.string_build(code)
         self.assertEqual(ast.as_string().strip(), code.strip())
 
-    # This test is disabled on PyPy because we cannot get a release that has proper
-    # support for f-strings (we need 7.2 at least)
-    @pytest.mark.skipif(
-        platform.python_implementation() == "PyPy",
-        reason="Needs f-string support.",
-    )
     def test_f_strings(self):
         code = r'''
 a = f"{'a'}"
@@ -561,7 +531,7 @@ from ..cave import wine\n\n"""
         """When we import PickleError from nonexistent, a call to the infer
         method of this From node will be made by unpack_infer.
         inference.infer_from will try to import this module, which will fail and
-        raise a InferenceException (by mixins.do_import_module). The infer_name
+        raise a InferenceException (by ImportNode.do_import_module). The infer_name
         will catch this exception and yield and Uninferable instead.
         """
 
@@ -879,9 +849,6 @@ class AnnAssignNodeTest(unittest.TestCase):
 
 
 class ArgumentsNodeTC(unittest.TestCase):
-    @pytest.mark.skip(
-        "FIXME  http://bugs.python.org/issue10445 (no line number on function args)"
-    )
     def test_linenumbering(self) -> None:
         ast = builder.parse(
             """
@@ -1030,13 +997,13 @@ class AliasesTest(unittest.TestCase):
             node.name = "another_test"
             return node
 
-        def test_callfunc(node: Call) -> Optional[Call]:
+        def test_callfunc(node: Call) -> Call | None:
             if node.func.name == "Foo":
                 node.func.name = "Bar"
                 return node
             return None
 
-        def test_assname(node: AssignName) -> Optional[AssignName]:
+        def test_assname(node: AssignName) -> AssignName | None:
             if node.name == "foo":
                 return nodes.AssignName(
                     "bar", node.lineno, node.col_offset, node.parent
@@ -1106,20 +1073,52 @@ class AliasesTest(unittest.TestCase):
 
 class Python35AsyncTest(unittest.TestCase):
     def test_async_await_keywords(self) -> None:
-        async_def, async_for, async_with, await_node = builder.extract_node(
+        (
+            async_def,
+            async_for,
+            async_with,
+            async_for2,
+            async_with2,
+            await_node,
+        ) = builder.extract_node(
             """
         async def func(): #@
             async for i in range(10): #@
                 f = __(await i)
             async with test(): #@
                 pass
+            async for i \
+                    in range(10):  #@
+                pass
+            async with test(), \
+                    test2():  #@
+                pass
         """
         )
-        self.assertIsInstance(async_def, nodes.AsyncFunctionDef)
-        self.assertIsInstance(async_for, nodes.AsyncFor)
-        self.assertIsInstance(async_with, nodes.AsyncWith)
-        self.assertIsInstance(await_node, nodes.Await)
-        self.assertIsInstance(await_node.value, nodes.Name)
+        assert isinstance(async_def, nodes.AsyncFunctionDef)
+        assert async_def.lineno == 2
+        assert async_def.col_offset == 0
+
+        assert isinstance(async_for, nodes.AsyncFor)
+        assert async_for.lineno == 3
+        assert async_for.col_offset == 4
+
+        assert isinstance(async_with, nodes.AsyncWith)
+        assert async_with.lineno == 5
+        assert async_with.col_offset == 4
+
+        assert isinstance(async_for2, nodes.AsyncFor)
+        assert async_for2.lineno == 7
+        assert async_for2.col_offset == 4
+
+        assert isinstance(async_with2, nodes.AsyncWith)
+        assert async_with2.lineno == 9
+        assert async_with2.col_offset == 4
+
+        assert isinstance(await_node, nodes.Await)
+        assert isinstance(await_node.value, nodes.Name)
+        assert await_node.lineno == 4
+        assert await_node.col_offset == 15
 
     def _test_await_async_as_string(self, code: str) -> None:
         ast_node = parse(code)
@@ -1227,7 +1226,7 @@ def test_type_comments_with() -> None:
         """
     with a as b: # type: int
         pass
-    with a as b: # type: ignore
+    with a as b: # type: ignore[name-defined]
         pass
     """
     )
@@ -1244,7 +1243,7 @@ def test_type_comments_for() -> None:
         """
     for a, b in [1, 2, 3]: # type: List[int]
         pass
-    for a, b in [1, 2, 3]: # type: ignore
+    for a, b in [1, 2, 3]: # type: ignore[name-defined]
         pass
     """
     )
@@ -1261,7 +1260,7 @@ def test_type_coments_assign() -> None:
     module = builder.parse(
         """
     a, b = [1, 2, 3] # type: List[int]
-    a, b = [1, 2, 3] # type: ignore
+    a, b = [1, 2, 3] # type: ignore[name-defined]
     """
     )
     node = module.body[0]
@@ -1595,23 +1594,36 @@ def test_assignment_expression_in_functiondef() -> None:
 
 
 def test_get_doc() -> None:
-    node = astroid.extract_node(
-        """
+    code = textwrap.dedent(
+        """\
     def func():
         "Docstring"
         return 1
     """
     )
-    assert node.doc == "Docstring"
+    node: nodes.FunctionDef = astroid.extract_node(code)  # type: ignore[assignment]
+    with pytest.warns(DeprecationWarning) as records:
+        assert node.doc == "Docstring"
+        assert len(records) == 1
+    assert isinstance(node.doc_node, nodes.Const)
+    assert node.doc_node.value == "Docstring"
+    assert node.doc_node.lineno == 2
+    assert node.doc_node.col_offset == 4
+    assert node.doc_node.end_lineno == 2
+    assert node.doc_node.end_col_offset == 15
 
-    node = astroid.extract_node(
-        """
+    code = textwrap.dedent(
+        """\
     def func():
         ...
         return 1
     """
     )
-    assert node.doc is None
+    node = astroid.extract_node(code)
+    with pytest.warns(DeprecationWarning) as records:
+        assert node.doc is None
+        assert len(records) == 1
+    assert node.doc_node is None
 
 
 @test_utils.require_version(minver="3.8")
