@@ -1,51 +1,36 @@
-# Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2011, 2013-2015 Google, Inc.
-# Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2013 Phil Schaf <flying-sheep@web.de>
-# Copyright (c) 2014 Eevee (Alex Munroe) <amunroe@yelp.com>
-# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
-# Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
-# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
-# Copyright (c) 2015 Philip Lorenz <philip@bithub.de>
-# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
-# Copyright (c) 2017, 2019 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2017-2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2017 Derek Gustafson <degustaf@gmail.com>
-# Copyright (c) 2018-2019 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2018 brendanator <brendan.maginnis@gmail.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2019-2021 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2019 Peter de Blanc <peter@standard.ai>
-# Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
-# Copyright (c) 2020 Tim Martin <tim@asymptotic.co.uk>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2021 Tushar Sadhwani <86737547+tushar-deepsource@users.noreply.github.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 doranid <ddandd@gmail.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
 """tests for specific behaviour of astroid scoped nodes (i.e. module, class and
 function)
 """
+
+from __future__ import annotations
+
 import datetime
 import os
 import sys
 import textwrap
 import unittest
+import warnings
 from functools import partial
-from typing import Any, List, Union
+from typing import Any
 
 import pytest
 
-from astroid import MANAGER, builder, nodes, objects, test_utils, util
+from astroid import (
+    MANAGER,
+    builder,
+    extract_node,
+    nodes,
+    objects,
+    parse,
+    test_utils,
+    util,
+)
 from astroid.bases import BoundMethod, Generator, Instance, UnboundMethod
-from astroid.const import PY38_PLUS
+from astroid.const import IS_PYPY, PY38, PY38_PLUS
 from astroid.exceptions import (
     AttributeInferenceError,
     DuplicateBasesError,
@@ -71,7 +56,7 @@ except ImportError:
 
 def _test_dict_interface(
     self: Any,
-    node: Union[nodes.ClassDef, nodes.FunctionDef, nodes.Module],
+    node: nodes.ClassDef | nodes.FunctionDef | nodes.Module,
     test_attr: str,
 ) -> None:
     self.assertIs(node[test_attr], node[test_attr])
@@ -202,8 +187,7 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
 
     def test_relative_to_absolute_name(self) -> None:
         # package
-        mod = nodes.Module("very.multi.package", "doc")
-        mod.package = True
+        mod = nodes.Module("very.multi.package", package=True)
         modname = mod.relative_to_absolute_name("utils", 1)
         self.assertEqual(modname, "very.multi.package.utils")
         modname = mod.relative_to_absolute_name("utils", 2)
@@ -213,8 +197,7 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
         modname = mod.relative_to_absolute_name("", 1)
         self.assertEqual(modname, "very.multi.package")
         # non package
-        mod = nodes.Module("very.multi.module", "doc")
-        mod.package = False
+        mod = nodes.Module("very.multi.module", package=False)
         modname = mod.relative_to_absolute_name("utils", 0)
         self.assertEqual(modname, "very.multi.utils")
         modname = mod.relative_to_absolute_name("utils", 1)
@@ -225,8 +208,7 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(modname, "very.multi")
 
     def test_relative_to_absolute_name_beyond_top_level(self) -> None:
-        mod = nodes.Module("a.b.c", "")
-        mod.package = True
+        mod = nodes.Module("a.b.c", package=True)
         for level in (5, 4):
             with self.assertRaises(TooManyLevelsError) as cm:
                 mod.relative_to_absolute_name("test", level)
@@ -280,7 +262,7 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
         path = resources.find("data/all.py")
         file_build = builder.AstroidBuilder().file_build(path, "all")
         with self.assertRaises(AttributeError):
-            # pylint: disable=pointless-statement
+            # pylint: disable=pointless-statement, no-member
             file_build.file_stream
 
     def test_stream_api(self) -> None:
@@ -292,6 +274,69 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
             with open(path, "rb") as file_io:
                 self.assertEqual(stream.read(), file_io.read())
 
+    @staticmethod
+    def test_singleline_docstring() -> None:
+        data = textwrap.dedent(
+            """\
+            '''Hello World'''
+            foo = 1
+        """
+        )
+        module = builder.parse(data, __name__)
+        assert isinstance(module.doc_node, nodes.Const)
+        assert module.doc_node.lineno == 1
+        assert module.doc_node.col_offset == 0
+        assert module.doc_node.end_lineno == 1
+        assert module.doc_node.end_col_offset == 17
+
+    @staticmethod
+    def test_multiline_docstring() -> None:
+        data = textwrap.dedent(
+            """\
+            '''Hello World
+
+            Also on this line.
+            '''
+            foo = 1
+        """
+        )
+        module = builder.parse(data, __name__)
+
+        assert isinstance(module.doc_node, nodes.Const)
+        assert module.doc_node.lineno == 1
+        assert module.doc_node.col_offset == 0
+        assert module.doc_node.end_lineno == 4
+        assert module.doc_node.end_col_offset == 3
+
+    @staticmethod
+    def test_comment_before_docstring() -> None:
+        data = textwrap.dedent(
+            """\
+            # Some comment
+            '''This is
+
+            a multiline docstring.
+            '''
+        """
+        )
+        module = builder.parse(data, __name__)
+
+        assert isinstance(module.doc_node, nodes.Const)
+        assert module.doc_node.lineno == 2
+        assert module.doc_node.col_offset == 0
+        assert module.doc_node.end_lineno == 5
+        assert module.doc_node.end_col_offset == 3
+
+    @staticmethod
+    def test_without_docstring() -> None:
+        data = textwrap.dedent(
+            """\
+            foo = 1
+        """
+        )
+        module = builder.parse(data, __name__)
+        assert module.doc_node is None
+
 
 class FunctionNodeTest(ModuleLoader, unittest.TestCase):
     def test_special_attributes(self) -> None:
@@ -442,6 +487,12 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         astroid = builder.parse("lmbd = lambda: None", __name__)
         self.assertEqual(f"{__name__}.<lambda>", astroid["lmbd"].parent.value.qname())
 
+    def test_lambda_getattr(self) -> None:
+        astroid = builder.parse("lmbd = lambda: None")
+        self.assertIsInstance(
+            astroid["lmbd"].parent.value.getattr("__code__")[0], nodes.Unknown
+        )
+
     def test_is_method(self) -> None:
         data = """
             class A:
@@ -473,6 +524,20 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         astroid = builder.parse(code, __name__)
         self.assertEqual(astroid["f"].argnames(), ["a", "b", "c", "args", "kwargs"])
 
+        code_with_kwonly_args = "def f(a, b, *args, c=None, d=None, **kwargs): pass"
+        astroid = builder.parse(code_with_kwonly_args, __name__)
+        self.assertEqual(
+            astroid["f"].argnames(), ["a", "b", "args", "c", "d", "kwargs"]
+        )
+
+    @unittest.skipUnless(PY38_PLUS, "positional-only argument syntax")
+    def test_positional_only_argnames(self) -> None:
+        code = "def f(a, b, /, c=None, *args, d, **kwargs): pass"
+        astroid = builder.parse(code, __name__)
+        self.assertEqual(
+            astroid["f"].argnames(), ["a", "b", "c", "args", "d", "kwargs"]
+        )
+
     def test_return_nothing(self) -> None:
         """test inferred value on a function with empty return"""
         data = """
@@ -555,6 +620,24 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         self.assertIsInstance(one, nodes.Const)
         self.assertEqual(one.value, 1)
 
+    def test_func_is_bound(self) -> None:
+        data = """
+        class MyClass:
+            def bound():  #@
+                pass
+        """
+        func = builder.extract_node(data)
+        self.assertIs(func.is_bound(), True)
+        self.assertEqual(func.implicit_parameters(), 1)
+
+        data2 = """
+        def not_bound():  #@
+            pass
+        """
+        func2 = builder.extract_node(data2)
+        self.assertIs(func2.is_bound(), False)
+        self.assertEqual(func2.implicit_parameters(), 0)
+
     def test_type_builtin_descriptor_subclasses(self) -> None:
         astroid = builder.parse(
             """
@@ -735,6 +818,118 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         self.assertIsInstance(inferred, nodes.ClassDef)
         self.assertEqual(inferred.name, "MyClass")
 
+    @staticmethod
+    def test_singleline_docstring() -> None:
+        code = textwrap.dedent(
+            """\
+            def foo():
+                '''Hello World'''
+                bar = 1
+        """
+        )
+        func: nodes.FunctionDef = builder.extract_node(code)  # type: ignore[assignment]
+
+        assert isinstance(func.doc_node, nodes.Const)
+        assert func.doc_node.lineno == 2
+        assert func.doc_node.col_offset == 4
+        assert func.doc_node.end_lineno == 2
+        assert func.doc_node.end_col_offset == 21
+
+    @staticmethod
+    def test_multiline_docstring() -> None:
+        code = textwrap.dedent(
+            """\
+            def foo():
+                '''Hello World
+
+                Also on this line.
+                '''
+                bar = 1
+        """
+        )
+        func: nodes.FunctionDef = builder.extract_node(code)  # type: ignore[assignment]
+
+        assert isinstance(func.doc_node, nodes.Const)
+        assert func.doc_node.lineno == 2
+        assert func.doc_node.col_offset == 4
+        assert func.doc_node.end_lineno == 5
+        assert func.doc_node.end_col_offset == 7
+
+    @staticmethod
+    def test_multiline_docstring_async() -> None:
+        code = textwrap.dedent(
+            """\
+            async def foo(var: tuple = ()):
+                '''Hello
+
+                World
+                '''
+        """
+        )
+        func: nodes.FunctionDef = builder.extract_node(code)  # type: ignore[assignment]
+
+        assert isinstance(func.doc_node, nodes.Const)
+        assert func.doc_node.lineno == 2
+        assert func.doc_node.col_offset == 4
+        assert func.doc_node.end_lineno == 5
+        assert func.doc_node.end_col_offset == 7
+
+    @staticmethod
+    def test_docstring_special_cases() -> None:
+        code = textwrap.dedent(
+            """\
+        def f1(var: tuple = ()):  #@
+            'Hello World'
+
+        def f2() -> "just some comment with an open bracket(":  #@
+            'Hello World'
+
+        def f3() -> "Another comment with a colon: ":  #@
+            'Hello World'
+
+        def f4():  #@
+            # It should work with comments too
+            'Hello World'
+        """
+        )
+        ast_nodes: list[nodes.FunctionDef] = builder.extract_node(code)  # type: ignore[assignment]
+        assert len(ast_nodes) == 4
+
+        assert isinstance(ast_nodes[0].doc_node, nodes.Const)
+        assert ast_nodes[0].doc_node.lineno == 2
+        assert ast_nodes[0].doc_node.col_offset == 4
+        assert ast_nodes[0].doc_node.end_lineno == 2
+        assert ast_nodes[0].doc_node.end_col_offset == 17
+
+        assert isinstance(ast_nodes[1].doc_node, nodes.Const)
+        assert ast_nodes[1].doc_node.lineno == 5
+        assert ast_nodes[1].doc_node.col_offset == 4
+        assert ast_nodes[1].doc_node.end_lineno == 5
+        assert ast_nodes[1].doc_node.end_col_offset == 17
+
+        assert isinstance(ast_nodes[2].doc_node, nodes.Const)
+        assert ast_nodes[2].doc_node.lineno == 8
+        assert ast_nodes[2].doc_node.col_offset == 4
+        assert ast_nodes[2].doc_node.end_lineno == 8
+        assert ast_nodes[2].doc_node.end_col_offset == 17
+
+        assert isinstance(ast_nodes[3].doc_node, nodes.Const)
+        assert ast_nodes[3].doc_node.lineno == 12
+        assert ast_nodes[3].doc_node.col_offset == 4
+        assert ast_nodes[3].doc_node.end_lineno == 12
+        assert ast_nodes[3].doc_node.end_col_offset == 17
+
+    @staticmethod
+    def test_without_docstring() -> None:
+        code = textwrap.dedent(
+            """\
+            def foo():
+                bar = 1
+        """
+        )
+        func: nodes.FunctionDef = builder.extract_node(code)  # type: ignore[assignment]
+        assert func.doc_node is None
+
 
 class ClassNodeTest(ModuleLoader, unittest.TestCase):
     def test_dict_interface(self) -> None:
@@ -764,7 +959,10 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             self.assertEqual(
                 len(cls.getattr("__doc__")), 1, (cls, cls.getattr("__doc__"))
             )
-            self.assertEqual(cls.getattr("__doc__")[0].value, cls.doc)
+            with pytest.warns(DeprecationWarning) as records:
+                self.assertEqual(cls.getattr("__doc__")[0].value, cls.doc)
+                assert len(records) == 1
+            self.assertEqual(cls.getattr("__doc__")[0].value, cls.doc_node.value)
             self.assertEqual(len(cls.getattr("__module__")), 4)
             self.assertEqual(len(cls.getattr("__dict__")), 1)
             self.assertEqual(len(cls.getattr("__mro__")), 1)
@@ -1078,15 +1276,19 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
                 print(x)
 
             @f(a=2,
-               b=3)
+               b=3,
+            )
             def g2():
                 pass
         """
         astroid = builder.parse(data)
         self.assertEqual(astroid["g1"].fromlineno, 4)
         self.assertEqual(astroid["g1"].tolineno, 5)
-        self.assertEqual(astroid["g2"].fromlineno, 9)
-        self.assertEqual(astroid["g2"].tolineno, 10)
+        if not PY38_PLUS or PY38 and IS_PYPY:
+            self.assertEqual(astroid["g2"].fromlineno, 9)
+        else:
+            self.assertEqual(astroid["g2"].fromlineno, 10)
+        self.assertEqual(astroid["g2"].tolineno, 11)
 
     def test_metaclass_error(self) -> None:
         astroid = builder.parse(
@@ -1350,11 +1552,11 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         assert len(slots) == 3, slots
         assert [slot.value for slot in slots] == ["a", "b", "c"]
 
-    def assertEqualMro(self, klass: nodes.ClassDef, expected_mro: List[str]) -> None:
+    def assertEqualMro(self, klass: nodes.ClassDef, expected_mro: list[str]) -> None:
         self.assertEqual([member.name for member in klass.mro()], expected_mro)
 
     def assertEqualMroQName(
-        self, klass: nodes.ClassDef, expected_mro: List[str]
+        self, klass: nodes.ClassDef, expected_mro: list[str]
     ) -> None:
         self.assertEqual([member.qname() for member in klass.mro()], expected_mro)
 
@@ -1497,10 +1699,12 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
                 "FinalClass",
                 "ClassB",
                 "MixinB",
-                "",
+                # We don't recognize what 'cls' is at time of .format() call, only
+                # what it is at the end.
+                # "strMixin",
                 "ClassA",
                 "MixinA",
-                "",
+                "intMixin",
                 "Base",
                 "object",
             ],
@@ -1525,7 +1729,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         assert isinstance(cls, nodes.ClassDef)
         self.assertEqualMro(cls, ["C", "A", "B", "object"])
 
-    @test_utils.require_version(minver="3.7")
     def test_mro_generic_1(self):
         cls = builder.extract_node(
             """
@@ -1541,7 +1744,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             cls, [".C", ".A", "typing.Generic", ".B", "builtins.object"]
         )
 
-    @test_utils.require_version(minver="3.7")
     def test_mro_generic_2(self):
         cls = builder.extract_node(
             """
@@ -1557,7 +1759,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             cls, [".C", ".A", ".B", "typing.Generic", "builtins.object"]
         )
 
-    @test_utils.require_version(minver="3.7")
     def test_mro_generic_3(self):
         cls = builder.extract_node(
             """
@@ -1574,7 +1775,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             cls, [".D", ".B", ".A", ".C", "typing.Generic", "builtins.object"]
         )
 
-    @test_utils.require_version(minver="3.7")
     def test_mro_generic_4(self):
         cls = builder.extract_node(
             """
@@ -1590,7 +1790,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             cls, [".C", ".A", ".B", "typing.Generic", "builtins.object"]
         )
 
-    @test_utils.require_version(minver="3.7")
     def test_mro_generic_5(self):
         cls = builder.extract_node(
             """
@@ -1607,7 +1806,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             cls, [".C", ".A", ".B", "typing.Generic", "builtins.object"]
         )
 
-    @test_utils.require_version(minver="3.7")
     def test_mro_generic_6(self):
         cls = builder.extract_node(
             """
@@ -1624,7 +1822,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             cls, [".C", ".A", ".Generic", ".B", "typing.Generic", "builtins.object"]
         )
 
-    @test_utils.require_version(minver="3.7")
     def test_mro_generic_7(self):
         cls = builder.extract_node(
             """
@@ -1642,7 +1839,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             cls, [".E", ".C", ".A", ".B", "typing.Generic", ".D", "builtins.object"]
         )
 
-    @test_utils.require_version(minver="3.7")
     def test_mro_generic_error_1(self):
         cls = builder.extract_node(
             """
@@ -1656,7 +1852,6 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         with self.assertRaises(DuplicateBasesError):
             cls.mro()
 
-    @test_utils.require_version(minver="3.7")
     def test_mro_generic_error_2(self):
         cls = builder.extract_node(
             """
@@ -1670,6 +1865,44 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         with self.assertRaises(DuplicateBasesError):
             cls.mro()
 
+    def test_mro_typing_extensions(self):
+        """Regression test for mro() inference on typing_extesnions.
+
+        Regression reported in:
+        https://github.com/PyCQA/astroid/issues/1124
+        """
+        module = parse(
+            """
+        import abc
+        import typing
+        import dataclasses
+
+        import typing_extensions
+
+        T = typing.TypeVar("T")
+
+        class MyProtocol(typing_extensions.Protocol): pass
+        class EarlyBase(typing.Generic[T], MyProtocol): pass
+        class Base(EarlyBase[T], abc.ABC): pass
+        class Final(Base[object]): pass
+        """
+        )
+        class_names = [
+            "ABC",
+            "Base",
+            "EarlyBase",
+            "Final",
+            "Generic",
+            "MyProtocol",
+            "Protocol",
+            "object",
+        ]
+        if not PY38_PLUS:
+            class_names.pop(-2)
+
+        final_def = module.body[-1]
+        self.assertEqual(class_names, sorted(i.name for i in final_def.mro()))
+
     def test_generator_from_infer_call_result_parent(self) -> None:
         func = builder.extract_node(
             """
@@ -2024,6 +2257,52 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         # Should not crash
         builder.parse(data)
 
+    @staticmethod
+    def test_singleline_docstring() -> None:
+        code = textwrap.dedent(
+            """\
+            class Foo:
+                '''Hello World'''
+                bar = 1
+        """
+        )
+        node: nodes.ClassDef = builder.extract_node(code)  # type: ignore[assignment]
+        assert isinstance(node.doc_node, nodes.Const)
+        assert node.doc_node.lineno == 2
+        assert node.doc_node.col_offset == 4
+        assert node.doc_node.end_lineno == 2
+        assert node.doc_node.end_col_offset == 21
+
+    @staticmethod
+    def test_multiline_docstring() -> None:
+        code = textwrap.dedent(
+            """\
+            class Foo:
+                '''Hello World
+
+                Also on this line.
+                '''
+                bar = 1
+        """
+        )
+        node: nodes.ClassDef = builder.extract_node(code)  # type: ignore[assignment]
+        assert isinstance(node.doc_node, nodes.Const)
+        assert node.doc_node.lineno == 2
+        assert node.doc_node.col_offset == 4
+        assert node.doc_node.end_lineno == 5
+        assert node.doc_node.end_col_offset == 7
+
+    @staticmethod
+    def test_without_docstring() -> None:
+        code = textwrap.dedent(
+            """\
+            class Foo:
+                bar = 1
+        """
+        )
+        node: nodes.ClassDef = builder.extract_node(code)  # type: ignore[assignment]
+        assert node.doc_node is None
+
 
 def test_issue940_metaclass_subclass_property() -> None:
     node = builder.extract_node(
@@ -2156,6 +2435,50 @@ def test_issue940_metaclass_funcdef_is_not_datadescriptor() -> None:
     assert isinstance(inferred, objects.Property)
 
 
+def test_property_in_body_of_try() -> None:
+    """Regression test for https://github.com/PyCQA/pylint/issues/6596."""
+    node: nodes.Return = builder._extract_single_node(
+        """
+    def myfunc():
+        try:
+
+            @property
+            def myfunc():
+                return None
+
+        except TypeError:
+            pass
+
+        @myfunc.setter
+        def myfunc():
+            pass
+
+        return myfunc() #@
+    """
+    )
+    next(node.value.infer())
+
+
+def test_property_in_body_of_if() -> None:
+    node: nodes.Return = builder._extract_single_node(
+        """
+    def myfunc():
+        if True:
+
+            @property
+            def myfunc():
+                return None
+
+        @myfunc.setter
+        def myfunc():
+            pass
+
+        return myfunc() #@
+    """
+    )
+    next(node.value.infer())
+
+
 def test_issue940_enums_as_a_real_world_usecase() -> None:
     node = builder.extract_node(
         """
@@ -2172,6 +2495,97 @@ def test_issue940_enums_as_a_real_world_usecase() -> None:
     assert sorted(actual) == ["bee", "cat"]
 
 
+def test_enums_type_annotation_str_member() -> None:
+    """A type-annotated member of an Enum class where:
+    - `member.value` is of type `nodes.Const` &
+    - `member.value.value` is of type `str`
+    is inferred as: `repr(member.value.value)`
+    """
+    node = builder.extract_node(
+        """
+    from enum import Enum
+    class Veg(Enum):
+        TOMATO: str = "sweet"
+
+    Veg.TOMATO.value
+    """
+    )
+    inferred_member_value = node.inferred()[0]
+    assert isinstance(inferred_member_value, nodes.Const)
+    assert inferred_member_value.value == "sweet"
+
+
+@pytest.mark.parametrize("annotation", ["bool", "dict", "int", "str"])
+def test_enums_type_annotation_no_value(annotation) -> None:
+    """A type-annotated member of an Enum class which has no value where:
+    - `member.value.value` is `None`
+    is not inferred
+    """
+    node = builder.extract_node(
+        """
+    from enum import Enum
+    class Veg(Enum):
+        TOMATO: {annotation}
+
+    Veg.TOMATO.value
+    """
+    )
+    inferred_member_value = node.inferred()[0]
+    assert inferred_member_value.value is None
+
+
+@pytest.mark.parametrize("annotation, value", [("int", 42), ("bytes", b"")])
+def test_enums_type_annotation_non_str_member(annotation, value) -> None:
+    """A type-annotated member of an Enum class where:
+    - `member.value` is of type `nodes.Const` &
+    - `member.value.value` is not of type `str`
+    is inferred as: `member.value.value`
+    """
+
+    node = builder.extract_node(
+        f"""
+    from enum import Enum
+    class Veg(Enum):
+        TOMATO: {annotation} = {value}
+
+    Veg.TOMATO.value
+    """
+    )
+    inferred_member_value = node.inferred()[0]
+    assert isinstance(inferred_member_value, nodes.Const)
+    assert inferred_member_value.value == value
+
+
+@pytest.mark.parametrize(
+    "annotation, value",
+    [
+        ("dict", {"variety": "beefeater"}),
+        ("list", ["beefeater", "moneymaker"]),
+        ("TypedDict", {"variety": "moneymaker"}),
+    ],
+)
+def test_enums_type_annotations_non_const_member(annotation, value) -> None:
+    """A type-annotated member of an Enum class where:
+    - `member.value` is not of type `nodes.Const`
+    is inferred as: `member.value.as_string()`.
+    """
+
+    member = builder.extract_node(
+        f"""
+    from enum import Enum
+
+    class Veg(Enum):
+        TOMATO: {annotation} = {value}
+
+    Veg.TOMATO.value
+    """
+    )
+
+    inferred_member_value = member.inferred()[0]
+    assert not isinstance(inferred_member_value, nodes.Const)
+    assert inferred_member_value.as_string() == repr(value)
+
+
 def test_metaclass_cannot_infer_call_yields_an_instance() -> None:
     node = builder.extract_node(
         """
@@ -2256,7 +2670,6 @@ def test_posonlyargs_default_value() -> None:
     assert first_param.value == 1
 
 
-@test_utils.require_version(minver="3.7")
 def test_ancestor_with_generic() -> None:
     # https://github.com/PyCQA/astroid/issues/942
     tree = builder.parse(
@@ -2352,5 +2765,85 @@ class TestFrameNodes:
         assert module.body[1].value.locals["x"][0].frame(future=True) == module
 
 
+def test_deprecation_of_doc_attribute() -> None:
+    code = textwrap.dedent(
+        """\
+    def func():
+        "Docstring"
+        return 1
+    """
+    )
+    node: nodes.FunctionDef = extract_node(code)  # type: ignore[assignment]
+    with pytest.warns(DeprecationWarning) as records:
+        assert node.doc == "Docstring"
+        assert len(records) == 1
+    with pytest.warns(DeprecationWarning) as records:
+        node.doc = None
+        assert len(records) == 1
+
+    code = textwrap.dedent(
+        """\
+    class MyClass():
+        '''Docstring'''
+    """
+    )
+    node: nodes.ClassDef = extract_node(code)  # type: ignore[assignment]
+    with pytest.warns(DeprecationWarning) as records:
+        assert node.doc == "Docstring"
+        assert len(records) == 1
+    with pytest.warns(DeprecationWarning) as records:
+        node.doc = None
+        assert len(records) == 1
+
+    code = textwrap.dedent(
+        """\
+    '''Docstring'''
+    """
+    )
+    node = parse(code)
+    with pytest.warns(DeprecationWarning) as records:
+        assert node.doc == "Docstring"
+        assert len(records) == 1
+    with pytest.warns(DeprecationWarning) as records:
+        node.doc = None
+        assert len(records) == 1
+
+    # If 'doc' isn't passed to Module, ClassDef, FunctionDef,
+    # no DeprecationWarning should be raised
+    doc_node = nodes.Const("Docstring")
+    with warnings.catch_warnings():
+        # Modify warnings filter to raise error for DeprecationWarning
+        warnings.simplefilter("error", DeprecationWarning)
+        node_module = nodes.Module(name="MyModule")
+        node_module.postinit(body=[], doc_node=doc_node)
+        assert node_module.doc_node == doc_node
+        node_class = nodes.ClassDef(name="MyClass")
+        node_class.postinit(bases=[], body=[], decorators=[], doc_node=doc_node)
+        assert node_class.doc_node == doc_node
+        node_func = nodes.FunctionDef(name="MyFunction")
+        node_func.postinit(args=nodes.Arguments(), body=[], doc_node=doc_node)
+        assert node_func.doc_node == doc_node
+
+    # Test 'doc' attribute if only 'doc_node' is passed
+    with pytest.warns(DeprecationWarning) as records:
+        assert node_module.doc == "Docstring"
+        assert len(records) == 1
+    with pytest.warns(DeprecationWarning) as records:
+        assert node_class.doc == "Docstring"
+        assert len(records) == 1
+    with pytest.warns(DeprecationWarning) as records:
+        assert node_func.doc == "Docstring"
+        assert len(records) == 1
+
+    # If 'doc' is passed to Module, ClassDef, FunctionDef,
+    # a DeprecationWarning should be raised
+    doc_node = nodes.Const("Docstring")
+    with pytest.warns(DeprecationWarning) as records:
+        node_module = nodes.Module(name="MyModule", doc="Docstring")
+        node_class = nodes.ClassDef(name="MyClass", doc="Docstring")
+        node_func = nodes.FunctionDef(name="MyFunction", doc="Docstring")
+        assert len(records) == 3
+
+
 if __name__ == "__main__":
     unittest.main()
